import timeit

from math import radians, exp
import numpy as np
import matplotlib.pyplot as plt

from RIaw import RIaw
from Cox_Munk_BERON import Cox_Munk_BERON
from Snells_Law import Snells_Law
from indexing import indexing
from betasw_ZHH2009 import betasw_ZHH2009
from inversion_2SeaColor import inversion_2SeaColor

## main
##Created by Suhyb Salama @ITC 2015
##Salama, M. S., & Verhoef, W. (2015). 
##Two-stream remote sensing model for water quality mapping: 2SeaColor.
##Remote sensing of environment, 157, 111-122. https://doi.org/10.1016/j.rse.2014.07.022

## updated 10 of March 2020 for EO4SD-Coastal project

def load(filename):
    result = []
    lines = open(filename, 'r').readlines()
    for line in lines:
        if len(line)>0 and line[0] != '%':
            line = line.split()
            line = [float(s) for s in line]
            result.append(line)
    return np.array(result)

# load column-wise

def load_c(filename):
    result = []
    lines = open(filename, 'r').readlines()
    for line in lines:
        if len(line)>0 and line[0] != '%':
            line = line.split()
            line = [float(s) for s in line]
            if len(result) == 0:
                result = [[f] for f in line]
            else:
                for i in range(0,len(result)):
                    result[i].append(line[i])
    return np.array(result)

def interp1(nx,nny,xx):
    result = np.array([np.interp(xx,nx,ny) for ny in nny])
    return result


class TwoSeaColor:
    def __init__(self,sza,vza,saa,vaa):
        self.z=[0]
        #self.wlB = np.array([443,490,560,665,705,740,783,842,865])  #S2 wavelength
        self.wlB = np.array([443,490,560,665,705,740,783,842])  #S2 wavelength B01..B08

        ##Water input

        self.Tc=25 # water tempreture 
        self.Q=3.25 # istropic factor
        self.Salinty=30
        self.ws=5 #Wind speed
        self.phi_w=15 # wind direction if possible read

        self.nw, self.na = RIaw(self.wlB,self.Tc,self.Salinty)

        self.Coef_chla=0.2
        self.Coef_dom=0.15 
        self.Coef_spm=0.018 
        self.Coef_y=0.5
        self.Coef_s=0.002  

        self.L_needed=np.array([443,490,560,665])
        indx_=indexing(self.L_needed,self.wlB)
        self.indx_440=indx_[0]
        self.indx_490=indx_[1]
        self.indx_550=indx_[2]
        if len(indx_)>3:
            self.indx_640=indx_[3]
        self.eta0=0.0182
        self.s=0.021
        self.ingamma=0.98 #0.80
        self.fsky=0 #0.5

        ##
        self.Aa0=load('A_mod.m')
        self.Aa0=self.Aa0.transpose()
        self.Aa=interp1(self.Aa0[0],self.Aa0[1:],self.wlB)
        self.AWi=load_c('aw.m') 
        self.AW_correctionsi=load_c('aw_Corrections.m') 

        ## #Röttgers, R. McKee, D, and Utschig, C, "Temperature and salinity correction coefficients for light absorption by water in the visible to infrared spectral region", Opt. Express, 2014			
        # Data of the salinity and temperture correction coefficients for the light absorption coefficent of water			
        self.aw_corrections=interp1(self.AW_correctionsi[0],self.AW_correctionsi[1:],self.wlB) # column 0 and columns 1,2,3

        self.aw0=interp1(self.AWi[0],[self.AWi[1]],self.wlB)[0]
        self.aw=self.aw0+(self.Tc-20)*self.aw_corrections[2]+self.Salinty*self.aw_corrections[0]

        self.phi=vaa-saa
        self.ra_w, _, _=Cox_Munk_BERON(self.ws,self.phi_w,sza,vza,saa,vaa,self.na,self.nw) 
        self.ta_w=1-self.ra_w
        self.vza_w=Snells_Law(vza,self.na,self.nw) 
        self.sza_w=Snells_Law(sza,self.na,self.nw) 
        self.rw_a, _, _=Cox_Munk_BERON(self.ws,self.phi_w,self.vza_w,radians(90)-vza,saa,vaa,self.nw,self.na)
        self.tw_a=1-self.rw_a

        ## Xiaodong Zhang, Lianbo Hu, and Ming-Xia He (2009), Scatteirng by pure
        # seawater: Effect of salinity, Optics Express, Vol. 17, No. 7, 5698-5710 
        _, _, bsw = betasw_ZHH2009(self.wlB,self.Tc,sza,self.Salinty)
        self.bbsw=0.5*bsw

        self.lb= [1e-5,  1e-05, 1e-05, 1e-05,   1e-05]
        self.ub= [0.65 ,  5,  0.35 , 0.025,    1.7]

    def process(self, obsR):
        #start = timeit.default_timer()

        # approx the phytoplankton absorption-coefficent at 440nm (Lee et al., 1998)
        dummy=0.072*(obsR[self.indx_440]/obsR[self.indx_550])**-1.62 
        a_chla = self.Coef_chla*dummy
        a_dg = self.Coef_dom*dummy 
        bb_SPM = self.Coef_spm*9.324*obsR[self.indx_640]  #combination of particles backscattering coefficient + viewing angle + sea state,at 400nm , Lee et al. 1999.
        Y = abs(self.Coef_y*3.44*(1-3.17*exp(-2.01*obsR[self.indx_440]/obsR[self.indx_490])))  #Y is between the range 0 - 2.5, Lee at al. 1996

        IOP0=np.array([a_chla,a_dg,bb_SPM,self.s,Y]) # [float,float,float,float,float]
        Rrs, R, Kd, IOP = inversion_2SeaColor(obsR,IOP0,self.lb,self.ub,self.eta0,self.ingamma,self.fsky,self.ta_w,self.tw_a, self.wlB,self.sza_w ,self.Q,self.nw,self.z,self.Aa,self.aw,self.bbsw)

        #stop = timeit.default_timer()
        #execution_time = stop - start

        #print("Program Executed in "+str(execution_time)) # It returns time in seconds

        return Rrs, R, Kd, IOP
